@@ -1,18 +1,24 @@
+----
+----
 --
--- When lumail launches it will look for two configuration files, if they
--- exist they will be loaded in turn:
+-- When lumail launches it will look for the following configuration files:
 --
 --   /etc/lumail.lua
+--   /etc/lumail.d/*.lua
 --   ~/.lumail/config.lua
 --
--- If you wish to load an additional configuration file you can add it via
--- the --rcfile argument:
+-- Each of these files will be loaded in turn.  If none are present then
+-- the client will abort with an error message.
 --
---   $ lumail --rcfile ./lumail.lua
+-- If you wish to load additional configuration file(s) you may do so via
+-- the "--rcfile" argument.  Multiple additional files may be specified:
 --
+--   $ lumail --rcfile ./one.lua --rcfile ./two.lua
 --
--- NOTE: This fille will also ensure that each file matching *.lua
---       beneath ~/.lumail.d/ will be loaded.
+-- NOTE: This file will also cause ~/.lumail.d/*.lua to be loaded.
+--
+---
+---
 --
 -- The configuration file(s) are responsible for determining which keystrokes
 -- are recognized, and what they will do.
@@ -23,12 +29,13 @@
 --
 --   http://lumail.org/examples/
 --
--- The lumail primitives are documented here:
+-- The lumail primitives themselves, from which the examples are built, are
+-- documented here:
 --
 --   http://lumail.org/lua/
 --
--- Steve
--- --
+----
+----
 --
 
 
@@ -43,10 +50,13 @@
 --
 prefix = os.getenv( "HOME" ) .. "/Maildir"
 
+--
+-- Ensure the directory exists.
+--
 if ( is_directory(prefix) ) then
    maildir_prefix(prefix)
 else
-   abort( "The specified Maildir prefix isn't a directory: " .. prefix );
+   abort( "The specified Maildir prefix isn't a directory: " .. prefix )
 end
 
 
@@ -54,21 +64,25 @@ end
 -- There is only one folder which is special, and that is the one where
 -- lumail will record copies of outgoing mail(s).
 --
--- The sent-mail folder may be specified here:
+-- The sent-mail folder may be specified here, if it exists:
 --
-sent_mail( maildir_prefix() .. "/sent-mail" );
+-- NOTE:  This is commented out by default.
+--
+-- if ( is_maildir( maildir_prefix() .. "/sent-mail" ) ) then
+--   sent_mail( maildir_prefix() .. "/sent-mail" )
+--end
 
 
 --
--- Set the from address for yourself, which is used for
--- composing/replying.
+-- Set the from address for yourself, which is used for composing/replying.
 --
 -- This can be changed in one of the available hooks.
 --
--- In this configuration file we use the `on_folder_selection()` function later.
+-- In this configuration file we use the `on_folder_selection()` function
+-- to change this on a per-folder basis.
 --
-default_email = "Steve Kemp <steve@steve.org.uk>";
-from( default_email );
+default_email = "Steve Kemp <steve@steve.org.uk>"
+from( default_email )
 
 
 --
@@ -77,12 +91,13 @@ from( default_email );
 -- Prefer /usr/sbin/sendmail, but use /usr/lib/sendmail if that exists
 --
 if ( executable( "/usr/sbin/sendmail" ) ) then
-   sendmail_path( "/usr/sbin/sendmail -t" );
+   sendmail_path( "/usr/sbin/sendmail -t" )
 end
 if ( executable( "/usr/lib/sendmail" ) ) then
-   sendmail_path( "/usr/lib/sendmail -t" );
+   sendmail_path( "/usr/lib/sendmail -t" )
 end
 
+
 --
 -- Specify the editor to use for composing/replying to email messages.
 --
@@ -113,12 +128,12 @@ end
 --
 --    all  -> Show all maildir folders.
 --    new  -> Show all maildir folders which contain unread messages.
---   "str" -> Show all maildir folders which match the substring "str".
+--   "pat" -> Show all maildir folders which match the regular expression "pat".
 --
 -- See "all_folders" , "new_folders", and "livejournal_folders" functions for
 -- example of use.
 --
-maildir_limit( "all" );
+maildir_limit( "all" )
 
 
 --
@@ -134,7 +149,7 @@ maildir_limit( "all" );
 --   $PATH   - The maildir path.
 --   $NAME   - The maildir name (i.e. dirname($PATH)).
 --
-maildir_format( "$CHECK - $UNREAD/$TOTAL - $PATH" );
+maildir_format( "$CHECK - $UNREAD/$TOTAL - $PATH" )
 
 
 --
@@ -144,9 +159,9 @@ maildir_format( "$CHECK - $UNREAD/$TOTAL - $PATH" );
 --
 --        all  -> Show all message.
 --        new  -> Show all unread messages.
---       "str" -> Show all messages which match the substring "str".
+--       "pat" -> Show all messages which match the regular expression "pat".
 --
-index_limit( "all" );
+index_limit( "all" )
 
 
 --
@@ -165,7 +180,7 @@ index_limit( "all" );
 --   $TO
 --
 --
-index_format( "[$FLAGS] $DAY/$MONTH/$YEAR $FROM - $SUBJECT" );
+index_format( "[$FLAGS] $DAY/$MONTH/$YEAR $FROM - $SUBJECT" )
 
 
 --
@@ -173,7 +188,22 @@ index_format( "[$FLAGS] $DAY/$MONTH/$YEAR $FROM - $SUBJECT" );
 --
 -- (The headers listed are shown in the order they are specified here.)
 --
-headers = { "$TO", "$FROM", "$DATE", "$SUBJECT" }
+headers = { "$TO", "$CC", "$FROM", "$DATE", "$SUBJECT" }
+
+
+--
+-- Choose the highlight mode for the selected column in maildir and index mode.
+-- Valid options are:
+--
+--   underline,standout,reverse,blink,dim,bold.
+--
+-- These settings might be useful if you run inside a terminal multiplexer such as tmux.
+--
+-- NOTE: You can use multiple values here:
+--
+index_highlight_mode('bold standout')
+maildir_highlight_mode('bold standout')
+
 
 
 --
@@ -185,30 +215,52 @@ headers = { "$TO", "$FROM", "$DATE", "$SUBJECT" }
 -- can make an email nicer to view, and it can be used with a
 -- setting such as this one:
 --
---     message_filter( "t-prot -cmekatlS --bigq --pgp-move-vrf -Mmutt -L/etc/t-prot/footers -A/etc/t-prot/ads" );
+--     display_filter( "t-prot -cmekatlS --bigq --pgp-move-vrf -Mmutt -L/etc/t-prot/footers -A/etc/t-prot/ads" )
 --
 -- The following would convert each message to be 100% upper-case
 --
---     message_filter( "tr '[:lower:]' '[:upper:]'" );
+--     display_filter( "tr '[:lower:]' '[:upper:]'" )
 --
 -- Note: There is no filter by default.
 --
 
 
+--
+-- This function will invoke offlineimap, if it is installed
+-- and configured for the current user.
+--
+-- It is called via the on_idle() function defined later.
+--
+function offlineimap()
+   if ( not file_exists( os.getenv( "HOME" ) .. "/.offlineimaprc" ) ) then
+      return false
+   end
+   if ( not executable( "/usr/bin/offlineimap" ) ) then
+      return false
+   end
+
+   exec( "/usr/bin/offlineimap" )
+   clear()
+   return true
+end
+
 --
 -- This function is called when the client is launched.
 --
 -- You might consider something useful like this:
 --
---    os.execute( "imapsync ..." );
+--    function on_start()
+--       offlineimap()
+--    end
 --
 function on_start()
-   msg("lumail v" .. VERSION .. " http://lumail.org/" );
+   msg("lumail v" .. VERSION .. " http://lumail.org/" )
 end
 
 
 --
--- This function is called when a folder is added/removed to the selected set.  Or toggled.
+-- This function is called when a folder is added/removed to the selected set.,
+-- or toggled.
 --
 -- It is where you can run per-folder hooks.
 --
@@ -220,31 +272,65 @@ function on_folder_selection( folder )
    -- Change the email address we use depending on our folder.
    --
    if ( string.find( folder, "debian-administration", 1, true )  ) then
-      from ("Steve <steve@debian-administration.org>");
+      from ("Steve <steve@debian-administration.org>")
    elseif ( string.find( folder, "debian" , 1, true )  ) then
-      from("Steve <skx@debian.org>");
+      from("Steve <skx@debian.org>")
    else
-      from( default_email );
+      from( default_email )
    end
 
-   msg( "There are " .. count_messages() .. " messages" );
+   msg( "There are " .. count_messages() .. " messages" )
 end
 
 
 --
--- Jump to the end of the list of messages/maildirs.
+-- Jump to the end of the screen, regardless of mode.
 --
 function jump_to_end()
 
    mode = global_mode()
-   mode = string.lower( mode );
+   mode = string.lower( mode )
 
    if ( string.find( mode, "maildir" ) ) then
-      jump_maildir_to( count_maildirs() - 1 );
+
+      -- Jump to last Maildir in the list.
+      jump_maildir_to( count_maildirs() - 1 )
+
    elseif (string.find(mode, "index" ) ) then
-      jump_index_to( count_messages() - 1 );
+
+      -- Jump to the last message in the list.
+      jump_index_to( count_messages() - 1 )
+
    else
-      msg( "jump_to_end not implemented for mode:" .. mode );
+
+      -- Jump to the end of the message.
+      scroll_message_to( count_lines() - 2 )
+   end
+end
+
+
+--
+-- Jump to the start of the screen, regardless of mode.
+--
+function jump_to_start()
+
+   mode = global_mode()
+   mode = string.lower( mode )
+
+   if ( string.find( mode, "maildir" ) ) then
+
+      -- Jump to the first Maildir
+      jump_maildir_to( 0 )
+
+   elseif (string.find(mode, "index" ) ) then
+
+      -- Jump to the first message.
+      jump_index_to( 0 )
+
+   else
+
+      -- Jump to the start of the message.
+      scroll_message_to( 0 )
    end
 end
 
@@ -258,20 +344,39 @@ end
 --
 -- (If the path were omitted then the current-message would be implicit.)
 --
--- We mark any new message(s) read when we read them here.
---
 function on_read_message( path )
    if ( is_new()  ) then
-      mark_read();
+      mark_read()
    end
 end
 
 
+
+--
+-- This is called if the user replies "no" to the "send message?"
+-- prompt which is issued at the end of the compose() and reply()
+-- functions.
+--
+function on_message_aborted( path )
+   msg( "Sending message aborted." )
+end
+
+
+--
+-- This function is called when a message is deleted.
+--
+-- You might save the message to ~/Maildir/.trash, or similar.
+--
+function on_delete_message( path )
+   msg( "Message deleted: " .. path )
+end
+
+
 --
 -- Show the version of this client.
 --
 function show_version()
-   msg("This is lumail version " .. VERSION );
+   msg("This is lumail version " .. VERSION .. " from http://lumail.org/" )
 end
 
 
@@ -283,26 +388,41 @@ end
 -- The function is expected to return a Lua table, the values of which
 -- will be used for completion.  (The keys being ignored.)
 --
--- Here we add any user-defined function the to the completion set.
--- This works because Lua uses the global "_G" table to store functions, etc.
+-- Here we add the known colours for the various colour-related primitives,
+-- as well as any user-defined function.
+--
+-- NOTE: User-functions come from the global Lua "_G" table.
 --
 --
 function on_complete()
-   ret = {}
 
+   --
+   -- Add the colours we support for the various display functions,
+   -- as well as the highlight options.
+   --
+   ret = {
+      "blue", "cyan", "green", "magenta", "red", "white", "yellow",
+      "blink", "bold", "dim", "reverse", "standout", "underline"
+   }
+
+   --
+   -- Add in all user-defined functions.
+   --
    for k,v in pairs(_G) do
       ret[k] = k
    end
+
    return(ret)
 end
 
+
 --
 -- This function is called when the client exits.
 --
--- It could also be used to run an imap-sync, or similar hook at exit-time.
+-- It could also be used to run a final imap-sync, or similar.
 --
 function on_exit()
-   print("Lumail has now terminated.");
+   print("Lumail has now terminated.")
    io.write( "That's it man, game over man, game over!\n")
 end
 
@@ -323,19 +443,22 @@ do
 
    function on_idle()
       m = global_mode()
-      m = string.lower( m );
+      m = string.lower( m )
 
       str = ""
 
       -- Set the message we'll display
       if ( string.find( m, "maildir" ) ) then
-         str = "mode:" .. m ..  " limit:" .. maildir_limit();
+         str = "mode:" .. m ..  " limit:" .. maildir_limit()
       else
-         str = "mode:" .. m ;
+         str = "mode:" .. m
       end
 
       -- Show the message & the time.
-      msg( str .. " time:" .. os.date("%X" ) );
+      msg( str .. " time:" .. os.date("%X" ) )
+
+      -- Dump the internal Lua-stack for debug-purposes.
+      dump_stack()
 
       --
       -- If the time between the last sync is more than
@@ -344,10 +467,10 @@ do
       ct = os.time()
       if ( ( ct - ls ) >=  ( 60 * 5 ) ) then
          ls = ct
-         if ( executable( "/usr/bin/imapsync" ) ) then
-            os.execute( "/usr/bin/imapsync" );
+         if ( offlineimap() ) then
+            msg( "offlineimap has synced your mail" )
          else
-            msg("/usr/bin/imapsync not installed" )
+            msg("offlineimap not available." )
          end
       end
    end
@@ -362,8 +485,94 @@ end
 -- (Selected folders will be displayed with a "[x]" next to them in maildir-mode.)
 --
 function index()
-   global_mode( "index" );
-   clear();
+    global_mode( "index" )
+    clear()
+end
+
+
+--
+-- This function will be called when the global mode changes
+--
+function on_mode_change( old, new)
+    -- check if we are switching from maildir to index mode, if so try to jump
+    -- the first unread mail or go to the last one
+    if string.find(old,'maildir') and string.find(new,'index') then
+        if (newmail_displayed()) then
+            jump_to_first_unread()
+        else
+            jump_to_end()
+        end
+    end
+end
+
+--
+-- Test if there is currently a new message displayed
+--
+function newmail_displayed()
+
+   --
+   -- save old limit + mode
+   --
+   -- See also: http://lumail.org/examples/state.html
+   --
+   old_index_limit = index_limit()
+   old_global_mode = global_mode()
+
+   -- check if we can count any new messages
+   index_limit( "new" )
+   global_mode( "index" )
+   c = count_messages()
+
+   -- restore
+   index_limit( old_index_limit )
+   global_mode( old_global_mode )
+
+   if ( c > 0 ) then
+      return true
+   else
+      return false
+   end
+end
+
+
+--
+-- Jump to the first unread mail starting from pos.
+--
+-- Do nothing if all mails are marked read
+--
+function jump_to_next_unread_from_pos(pos)
+   cur = index_offset()
+   count = count_messages()
+   i = pos
+   while( i < count ) do
+      jump_index_to( i )
+      if is_new() then
+         break
+      end
+      i = i + 1
+   end
+   if i == count then
+      jump_index_to(cur)
+   end
+end
+
+
+--
+-- This function will jump to the first unread message. If all messages are read
+-- to nothing
+--
+function jump_to_first_unread()
+   jump_to_next_unread_from_pos(0)
+end
+
+
+--
+-- This  function will jump to the next unread message.
+-- If all are read do nothing
+--
+function jump_to_next_unread()
+   cur = index_offset()
+   jump_to_next_unread_from_pos(cur+1)
 end
 
 
@@ -371,8 +580,8 @@ end
 -- Switch to the maildir-mode.
 --
 function maildir()
-   global_mode( "maildir" );
-   clear();
+   global_mode( "maildir" )
+   clear()
 end
 
 
@@ -380,8 +589,8 @@ end
 -- Move to message-mode with the currently selected message.
 --
 function view_message()
-   global_mode( "message");
-   clear();
+   global_mode( "message")
+   clear()
 end
 
 
@@ -389,8 +598,8 @@ end
 -- When in maildir-mode show all folders.
 --
 function all_folders()
-   maildir_limit( "all" );
-   clear();
+   maildir_limit( "all" )
+   clear()
 end
 
 
@@ -398,17 +607,18 @@ end
 -- When in maildir-mode show all folders which contain unread messages.
 --
 function new_folders()
-   maildir_limit( "new" );
-   clear();
+   maildir_limit( "new" )
+   clear()
 end
 
 
 --
--- When in maildir-mode show all folders which have a path matching the given pattern.
+-- When in maildir-mode show all folders which have a path matching
+-- the given regular expression.
 --
 function livejournal_folders()
-   maildir_limit( "livejournal.2" );
-   clear();
+   maildir_limit( "livejournal.[0-9]" )
+   clear()
 end
 
 
@@ -416,18 +626,18 @@ end
 -- Navigation functions for maildir-mode
 --
 function maildir_down()
-   scroll_maildir_down( 1 );
+   scroll_maildir_down( 1 )
 end
 function maildir_page_down()
    -- The minus-two is to account for the status-area
-   scroll_maildir_down( screen_height() - 2  );
+   scroll_maildir_down( screen_height() - 2  )
 end
 function maildir_up()
-   scroll_maildir_up(1);
+   scroll_maildir_up(1)
 end
 function maildir_page_up()
    -- The minus-two is to account for the status-area
-   scroll_maildir_up( screen_height() -2 );
+   scroll_maildir_up( screen_height() -2 )
 end
 
 
@@ -436,38 +646,81 @@ end
 -- Navigation functions for index-mode
 --
 function message_down()
-   scroll_index_down( 1 );
+   scroll_index_down( 1 )
 end
 function message_page_down()
    -- The minus-two is to account for the status-area
-   scroll_index_down( screen_height() - 2 );
+   scroll_index_down( screen_height() - 2 )
 end
 function message_up()
-   scroll_index_up(1);
+   scroll_index_up(1)
 end
 function message_page_up()
    -- The minus-two is to account for the status-area
-   scroll_index_up( screen_height() - 2 );
+   scroll_index_up( screen_height() - 2 )
 end
 
 
 --
--- Search for the next folder/message which matches the entered pattern.
+-- Should we wrap lines when showing a message?
 --
-function search_next()
-   x = prompt("/:" );
-   m = global_mode();
+function toggle_wrap_lines ()
+    if wrap_lines == nil then
+        wrap_lines = true
+    elseif wrap_lines == true then
+        wrap_lines = false
+    elseif wrap_lines == false then
+        wrap_lines = true
+    end
+end
 
-   if ( string.find( m, "maildir" ) ) then
-      scroll_maildir_to( x );
-   elseif (string.find(m, "index" ) ) then
-      scroll_index_to( x );
-   else
-      msg( "search_next() not implemented for mode:" .. m );
+
+
+
+
+do
+   --
+   -- This variable is used to allow searches to reuse prior input.
+   --
+   local search_next_prev = ''
+
+   --
+   -- Search for the next folder/message which matches the entered pattern.
+   --
+   function search_next()
+
+      --
+      -- The search pattern.
+      --
+      x = prompt("/:" )
+
+      --
+      -- If the user didn't enter anything then use the previous input.
+      --
+      if ( x == "" ) then x = search_next_prev end
+
+      --
+      -- Save the input for the next search.
+      --
+      search_next_prev = x
+
+      --
+      --  Get the mode, so we can do the correct kind of searching.
+      --
+      m = global_mode()
+
+      if ( string.find( m, "maildir" ) ) then
+         scroll_maildir_to( x )
+      elseif (string.find(m, "index" ) ) then
+         scroll_index_to( x )
+      else
+         msg( "search_next() not implemented for mode:" .. m )
+      end
    end
 end
 
 
+
 --
 -- Open a folder exclusively.
 --
@@ -475,10 +728,9 @@ end
 -- client.  I like to open multiple folders at once, because that's how I roll.
 --
 function open_folder()
-   clear_selected_folders();
+   clear_selected_folders()
    add_selected_folder()
-   global_mode( "index" );
-   clear();
+   index()
 end
 
 
@@ -487,16 +739,16 @@ end
 -- Open my most important folders.
 --
 function faves()
-   global_mode( "index" );
+   global_mode( "index" )
 
    -- ensure all folders are available
-   maildir_limit( "all" );
+   maildir_limit( "all" )
 
    -- clear the currently selected folders.
-   clear_selected_folders();
+   clear_selected_folders()
 
    -- The folders we care about
-   interesting = { "/.steve.org.uk", "/.people.kirsi",  "/.edinburgh-portraits.com" };
+   interesting = { "/.steve.org.uk", "/.people.kirsi",  "/.edinburgh-portraits.com" }
 
    -- For each one we're going to open
    for index,name in ipairs( interesting ) do
@@ -505,7 +757,7 @@ function faves()
       jump_maildir_to( 0 )
 
       -- Scroll forward until we find the matching folder
-      scroll_maildir_to( name );
+      scroll_maildir_to( name )
 
       --
       -- Note: "current_maildir" will return the value we've found.
@@ -520,8 +772,8 @@ function faves()
    end
 
    -- Now we've selected our folders, open them.
-   global_mode( "index" );
-   clear();
+   global_mode( "index" )
+   clear()
 end
 
 
@@ -542,14 +794,14 @@ end
 --
 -- Return contents of the given file.
 --
--- Note: Called by on_signature
+-- Note: Called by get_signature()
 --
 function file_contents(file)
     local f = io.open(file, "rb")
     if ( f ) then
        local content = f:read("*all")
        f:close()
-       return( content );
+       return( content )
     else
        return( "Error reading file: \"" .. file .. "\"" )
     end
@@ -628,6 +880,30 @@ function get_signature( from, to, subject )
 end
 
 
+--
+-- If the function "on_reply_transform_subject()" is defined it will
+-- be invoked when a message is replied to.
+--
+-- The intention is that you return an updated subject.
+--
+function on_reply_transform_subject( sub )
+
+   if ( string.len( sub ) < 1 ) then
+      return "No subject"
+   end
+   --
+   -- Remove (repeated) "Re:" from the start of string.
+   --
+   while( string.find(sub, "^[rR][eE]:" ) ) do
+      sub = string.gsub( sub, "^[rR][eE]:[ \t]+", "" )
+   end
+
+   -- Add prefix
+   return "Re: " .. sub
+end
+
+
+
 
 --
 -- Now setup keymaps for the different modes.
@@ -638,7 +914,7 @@ end
 -- However to avoid duplication there is a global keymap.
 --
 --   For example this works in all modes:
---      kemymap['global']['Q'] = 'exit()';
+--      kemymap['global']['Q'] = 'exit()'
 --
 keymap = {}
 keymap['global']  = {}
@@ -655,22 +931,31 @@ keymap['global']['Q'] = "exit()"
 -- Compose a new message.
 keymap['global']['c'] = "compose()"
 -- repl-mode
-keymap['global'][':'] = "loadstring(prompt(\":\"))();";
+keymap['global'][':'] = "loadstring(prompt(\":\"))()"
 -- Execute-shell
-keymap['global']['!'] = "exec(prompt(\"Shell command:\"));";
+keymap['global']['!'] = "exec(prompt(\"Shell command:\"))"
 -- Move to Maildir-mode
-keymap['global']['M']   = 'maildir()';
+keymap['global']['M']   = 'maildir()'
 -- Move to Index-mode
-keymap['global']['I'] = 'index()';
+keymap['global']['I'] = 'index()'
 -- Show the client version
 keymap['global']['v'] = 'show_version()'
 -- Jump to end
-keymap['global']['*'] = 'jump_to_end()'
+keymap['global']['*']        = 'jump_to_end()'
+keymap['global']['KEY_END']  = 'jump_to_end()'
+keymap['global']['KEY_HOME'] = 'jump_to_start()'
 -- Refresh display
-keymap['global']['^R'] = 'refresh_display()'
-keymap['global']['^L'] = 'refresh_display()'
+keymap['global']['^R'] = 'clear()'
+keymap['global']['^L'] = 'clear()'
+keymap['global']['KEY_RESIZE'] = 'clear()'
 
 
+--
+-- THIS WORKS :D.
+--
+--
+keymap['global']['Ѫ'] = 'msg("UTF-8 rocks!")'
+
 --
 -- Quitting means different things in different modes.
 --
@@ -679,7 +964,7 @@ keymap['message']['q'] = "index()"
 --  If viewing a folder quit means return to the maildir list.
 keymap['index']['q'] = "maildir()"
 -- If viewing a maildir list then q means exit.
-keymap['maildir']['q'] = "exit();"
+keymap['maildir']['q'] = "exit()"
 
 
 --
@@ -688,14 +973,20 @@ keymap['maildir']['q'] = "exit();"
 keymap['maildir']['a'] = 'all_folders()'
 keymap['maildir']['n'] = 'new_folders()'
 keymap['maildir']['l'] = 'livejournal_folders()'
+keymap['maildir']['g'] = 'maildir_limit( "(Google.*INBOX|Google.*All Mail)" )'
+
 
 --
 -- Scroll up/down & find folders
 --
 keymap['maildir']['j'] = 'maildir_down()'
+keymap['maildir']['KEY_DOWN'] = 'maildir_down()'
 keymap['maildir']['J'] = 'maildir_page_down()'
+keymap['maildir']['KEY_NPAGE'] = 'maildir_page_down()'
 keymap['maildir']['k'] = 'maildir_up()'
+keymap['maildir']['KEY_UP'] = 'maildir_up()'
 keymap['maildir']['K'] = 'maildir_page_up()'
+keymap['maildir']['KEY_PPAGE'] = 'maildir_page_up()'
 keymap['maildir']['/'] = 'search_next()'
 keymap['maildir']['f'] = 'faves()'
 
@@ -704,21 +995,32 @@ keymap['maildir']['f'] = 'faves()'
 -- Scroll up/down the message index
 --
 keymap['index']['j'] = 'message_down()'
+keymap['index']['KEY_DOWN'] = 'message_down()'
 keymap['index']['J'] = 'message_page_down()'
+keymap['index']['KEY_NPAGE'] = 'message_page_down()'
 keymap['index']['k'] = 'message_up()'
+keymap['index']['KEY_UP'] = 'message_up()'
 keymap['index']['K'] = 'message_page_up()'
+keymap['index']['KEY_PPAGE'] = 'message_page_up()'
 keymap['index']['/'] = 'search_next()'
 
 keymap['index']['Space'] = 'view_message()'
 keymap['index']['Enter'] = 'view_message()'
 keymap['index']['r'] = 'reply()'
+keymap['index']['f'] = 'forward()'
 keymap['index']['d'] = 'delete()'
 
 --
 -- View all/new messages only.
 --
-keymap['index']['a'] = 'index_limit("all");'
-keymap['index']['n'] = 'index_limit("new");'
+keymap['index']['a'] = 'index_limit("all")'
+keymap['index']['n'] = 'index_limit("new")'
+
+--
+-- Jump to the next unread mail in index or message mode
+--
+keymap['index']['N'] = 'jump_to_next_unread()'
+keymap['message']['N'] = 'jump_to_next_unread()'
 
 --
 -- Selection bindings.
@@ -737,7 +1039,11 @@ keymap['maildir']['Enter'] = 'open_folder()'
 --
 -- Scroll up/down in the current message.
 keymap['message']['j'] = 'scroll_message_down(1)'
+keymap['message']['KEY_DOWN'] = 'scroll_message_down(1)'
+keymap['message']['KEY_NPAGE'] = 'scroll_message_down(10)'
 keymap['message']['k'] = 'scroll_message_up(1)'
+keymap['message']['KEY_UP'] = 'scroll_message_up(1)'
+keymap['message']['KEY_PPAGE'] = 'scroll_message_up(10)'
 keymap['message']['Space'] = 'scroll_message_down(10)'
 
 --
@@ -746,9 +1052,13 @@ keymap['message']['Space'] = 'scroll_message_down(10)'
 keymap['message']['J'] = 'message_down()'
 keymap['message']['K'] = 'message_up()'
 
+-- wrap linese
+keymap['message']['w'] = 'toggle_wrap_lines()'
+
 -- Actions
-keymap['message']['r'] = 'reply()'
 keymap['message']['d'] = 'delete()'
+keymap['message']['f'] = 'forward()'
+keymap['message']['r'] = 'reply()'
 
 
 
@@ -765,41 +1075,59 @@ if ( is_directory( os.getenv( "HOME" ) .. "/.lumail.d" ) ) then
 end
 
 
+
+--
+-- Store persistant history.
+--
+history_file( os.getenv( "HOME" ) .. "/.lumail.history" )
+
 ---
 --
---   Further examples are available online:  http://lumail.org/examples/
+--   Further primitives which are not included here are documented
+--  online:
+--
+--       http://lumail.org/lua/
+--
+--
+--   Larger examples are also available online:
+--
+--       http://lumail.org/examples/
+--
+--
 --
 ---
 
---
--- Colour support for the display.
---
--- Valid colours: blue, cyan, green, magenta, red, white, yellow
---
 
 --
--- The colour of unread mailboxes in maildir-mode
-unread_maildir_colour( "red" );
+-- Jump between showing the full-path to all Maildirs
+-- and just the short-version.
+--
+function toggle_maildir_names()
+
+   -- get the current format-string.
+   fmt = maildir_format()
+
+   -- if it contains the short name
+   if ( fmt:find( "$NAME", 1, true ) ) then
+      -- replace it with the complete path.
+      fmt = fmt:gsub( "$NAME", "$PATH" )
+   else
+      -- otherwise if it contains the path
+      if ( fmt:find( "$PATH", 1, true ) ) then
+         -- replace with the short-name.
+         fmt = fmt:gsub( "$PATH", "$NAME" )
+      end
+   end
+
+   --
+   -- Update the format.
+   --
+   maildir_format( fmt )
+end
 
 --
--- The colour of unread messages in index-mode.
-unread_message_colour( "red" );
-
+-- Bind to a key.
 --
--- The colour of the attachment-list in message-mode.
-attachment_colour( "yellow" );
-
---
--- The colour of the headers in message-mode.
-header_colour( "cyan" );
-
---
--- The colour of the message-bod in message-mode
-body_colour( "green" );
+keymap['global']['p'] = "toggle_maildir_names()"
 
 
-
---
--- Folders to ignore in maildir mode
---
-ignored_folders = { "foo.", "bar." };
\ No newline at end of file
